-- Find closest target whose name matches the provided string.
--
-- Basic usage:
--
-- In your Lua console (press backtick) type:
--
--     /closest Orun
--
-- to target the nearest Orun Collector, for example.
--
-- Intermediate usage:
--
-- If you have something you always want to target, like Arklan Guardians, then
-- create an alias and then bind it to a key (such as 'A', for "Arklan").
-- Again, in your Lua console:
--
--     /alias target_closest_arkie "closest Arklan"
--     /bind A target_closest_arkie
--
-- Advanced usage:
--
-- Have the UI ask you what you want to target.  Once more, in the Lua console:
--
--     /alias ask_closest "prompt closest"
--     /bind C ask_closest
--
-- Now hit the 'C' key, and a prompt should show up at the bottom of your chat
-- area.  Type whatever you want to target, and it should automatically find the
-- closest and target it for you.

declare("closest", {})

function closest.use_targetless()
	if pcall(function() return targetless end) then
		return true
	else
		return false
	end
end

function closest:scan(args)
	local name = nil

	if args == nil or #args < 1 then
		name = closest.last_name
	else
		name = args[1]
		closest.last_name = name
	end

	if name == nil then
		print("Please specify what sort of thing to target (/closest <name>)")
		return
	end

	if closest.use_targetless() then
		targetless.api.radarlock = true
		targetless.var.lock = true
	end

	local best_tid, best_id, best_dist = nil, nil, 50000

	gkinterface.GKProcessCommand("RadarNone")
	gkinterface.GKProcessCommand("RadarNext")

	repeat
		local target_name = HUD.targetname.title or ""
		console_print("Looking at '"..target_name.."' for '"..name.."' ("..(string.find(target_name, name) or "nil")..")")
		if string.find(target_name, name) then
			if GetTargetDistance() < best_dist then
				console_print("DOONK")
				best_dist = GetTargetDistance()
				best_tid, best_id = radar.GetRadarSelectionID()
			end
		end

		gkinterface.GKProcessCommand("RadarNext")
	until radar.GetRadarSelectionID() == nil

	gkinterface.GKProcessCommand("RadarNone")

	if closest.use_targetless() then
		targetless.api.radarlock = false
		targetless.var.lock = false
	end

	if best_tid == nil then
		print("\127FF0000Nothing in range named '"..name.."'")
	else
		radar.SetRadarSelection(best_tid, best_id)
	end
end

RegisterUserCommand("closest", closest.scan)
